@@ -6,20 +6,17 @@
 
 #include "proxymodeltest.h"
 
-#include "dynamictreemodel.h"
-
+#include <QMimeData>
 #include <QSortFilterProxyModel>
 
-#include "modelspy.h"
-
 ProxyModelTest::ProxyModelTest(QObject *parent)
-    : QObject(parent),
-      m_rootModel(new DynamicTreeModel(this)),
-      m_sourceModel(m_rootModel),
-      m_proxyModel(nullptr),
-      m_intermediateProxyModel(nullptr),
-      m_modelSpy(new ModelSpy(this)),
-      m_modelCommander(new ModelCommander(m_rootModel, this))
+    : QObject(parent)
+    , m_rootModel(new DynamicTreeModel(this))
+    , m_sourceModel(m_rootModel)
+    , m_proxyModel(nullptr)
+    , m_intermediateProxyModel(nullptr)
+    , m_modelSpy(new ModelSpy(this))
+    , m_modelCommander(new ModelCommander(m_rootModel, this))
 {
 }
 
@@ -67,12 +64,19 @@ void ProxyModelTest::initRootModel(DynamicTreeModel *rootModel, const QString &c
     QMetaObject::invokeMethod(m_modelCommander, QString("init_" + currentTest).toLatin1(), Q_ARG(QString, currentTag));
 }
 
+static QSet<QString> unmatchedItems(const QStringList &list, const QStringList &items)
+{
+    const QSet<QString> itemsAset(items.constBegin(), items.constEnd());
+    const QSet<QString> listAsSet(list.constBegin(), list.constEnd());
+    return listAsSet - itemsAset;
+}
+
 void ProxyModelTest::verifyExecutedTests()
 {
     if (m_dataTags.contains(ProxyModelTestData::failTag())) {
         return;
     }
-    const QSet<QString> unimplemented = m_modelCommanderTags.toSet().subtract(m_dataTags.toSet());
+    const QSet<QString> unimplemented = unmatchedItems(m_modelCommanderTags, m_dataTags);
     QString unimplementedTestsString(QStringLiteral("("));
     for (const QString &test : unimplemented) {
         unimplementedTestsString.append(test + ",");
@@ -221,7 +225,7 @@ void ProxyModelTest::handleSignal(QVariantList expected)
         break;
     }
     case DataChanged: {
-        QVERIFY(expected.size() == 2);
+        QCOMPARE(expected.size(), 2);
         IndexFinder topLeftFinder = qvariant_cast<IndexFinder>(expected.at(0));
         topLeftFinder.setModel(m_proxyModel);
         QModelIndex topLeft = topLeftFinder.getIndex();
@@ -240,7 +244,6 @@ void ProxyModelTest::handleSignal(QVariantList expected)
         QCOMPARE(qvariant_cast<QModelIndex>(result.at(0)), topLeft);
         QCOMPARE(qvariant_cast<QModelIndex>(result.at(1)), bottomRight);
     }
-
     }
 }
 
@@ -290,16 +293,15 @@ void ProxyModelTest::testSourceReset()
     ins->setStartRow(0);
     ins->interpret(
         QLatin1String("- 1"
-        "- 2"
-        "- - 3"
-        "- - 4"
-        "- 5"
-        "- 6"
-        "- 7"
-        "- 8"
-        "- 9"
-        "- - 10")
-    );
+                      "- 2"
+                      "- - 3"
+                      "- - 4"
+                      "- 5"
+                      "- 6"
+                      "- 7"
+                      "- 8"
+                      "- 9"
+                      "- - 10"));
     ins->doCommand();
     // The proxymodel should reset any internal state it holds when the source model is reset.
     QPersistentModelIndex pmi = m_proxyModel->index(0, 0);
@@ -321,15 +323,14 @@ void ProxyModelTest::testDestroyModel()
     ins->setStartRow(0);
     ins->interpret(
         QLatin1String(" - 1"
-        " - 1"
-        " - - 1"
-        " - 1"
-        " - 1"
-        " - 1"
-        " - 1"
-        " - 1"
-        " - - 1")
-    );
+                      " - 1"
+                      " - - 1"
+                      " - 1"
+                      " - 1"
+                      " - 1"
+                      " - 1"
+                      " - 1"
+                      " - - 1"));
     ins->doCommand();
 
     QAbstractProxyModel *proxyModel = getProxy();
@@ -337,12 +338,20 @@ void ProxyModelTest::testDestroyModel()
 
     if (proxyModel->hasChildren()) {
         m_modelSpy->startSpying();
+        const bool prevPersistMode = m_modelSpy->useLazyPersistence();
+        m_modelSpy->setLazyPersistence(false); // don't persist on model reset during destruction
+        QCOMPARE(m_modelSpy->size(), 0);
         delete m_sourceModel;
         m_sourceModel = nullptr;
         m_modelSpy->stopSpying();
+        m_modelSpy->setLazyPersistence(prevPersistMode);
         testMappings();
-//     QCOMPARE(m_modelSpy->size(), 1);
-//     QVERIFY(m_modelSpy->takeFirst().first() == ModelReset);
+        // QItemSelectionModel in Qt6 signals a source model change if the source is destroyed
+        QVERIFY(m_modelSpy->size() == 0 || m_modelSpy->size() == 2);
+        if (m_modelSpy->size() == 2) {
+            QVERIFY(m_modelSpy->takeFirst().first() == ModelAboutToBeReset);
+            QVERIFY(m_modelSpy->takeFirst().first() == ModelReset);
+        }
     }
     m_sourceModel = currentSourceModel;
 }
@@ -407,51 +416,38 @@ void ProxyModelTest::verifyModel(const QModelIndex &topLeft, const QModelIndex &
 void ProxyModelTest::connectProxy(QAbstractProxyModel *proxyModel)
 {
     if (m_proxyModel) {
-        disconnect(m_proxyModel, SIGNAL(rowsAboutToBeInserted(QModelIndex,int,int)),
-                   this, SLOT(testMappings()));
-        disconnect(m_proxyModel, SIGNAL(rowsInserted(QModelIndex,int,int)),
-                   this, SLOT(testMappings()));
-        disconnect(m_proxyModel, SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)),
-                   this, SLOT(testMappings()));
-        disconnect(m_proxyModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-                   this, SLOT(testMappings()));
-        disconnect(m_proxyModel, SIGNAL(layoutAboutToBeChanged()),
-                   this, SLOT(testMappings()));
-        disconnect(m_proxyModel, SIGNAL(layoutChanged()),
-                   this, SLOT(testMappings()));
-        disconnect(m_proxyModel, SIGNAL(rowsAboutToBeMoved(QModelIndex,int,int,QModelIndex,int)),
-                   this, SLOT(testMappings()));
-        disconnect(m_proxyModel, SIGNAL(rowsMoved(QModelIndex,int,int,QModelIndex,int)),
-                   this, SLOT(testMappings()));
-        disconnect(m_proxyModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-                   this, SLOT(testMappings()));
+        disconnect(m_proxyModel, SIGNAL(rowsAboutToBeInserted(QModelIndex, int, int)), this, SLOT(testMappings()));
+        disconnect(m_proxyModel, SIGNAL(rowsInserted(QModelIndex, int, int)), this, SLOT(testMappings()));
+        disconnect(m_proxyModel, SIGNAL(rowsAboutToBeRemoved(QModelIndex, int, int)), this, SLOT(testMappings()));
+        disconnect(m_proxyModel, SIGNAL(rowsRemoved(QModelIndex, int, int)), this, SLOT(testMappings()));
+        disconnect(m_proxyModel, SIGNAL(layoutAboutToBeChanged()), this, SLOT(testMappings()));
+        disconnect(m_proxyModel, SIGNAL(layoutChanged()), this, SLOT(testMappings()));
+        disconnect(m_proxyModel, SIGNAL(rowsAboutToBeMoved(QModelIndex, int, int, QModelIndex, int)), this, SLOT(testMappings()));
+        disconnect(m_proxyModel, SIGNAL(rowsMoved(QModelIndex, int, int, QModelIndex, int)), this, SLOT(testMappings()));
+        disconnect(m_proxyModel, SIGNAL(dataChanged(QModelIndex, QModelIndex)), this, SLOT(testMappings()));
 
-        disconnect(m_proxyModel, SIGNAL(rowsAboutToBeInserted(QModelIndex,int,int)),
-                   this, SLOT(verifyModel(QModelIndex,int,int)));
-        disconnect(m_proxyModel, SIGNAL(rowsInserted(QModelIndex,int,int)),
-                   this, SLOT(verifyModel(QModelIndex,int,int)));
-        disconnect(m_proxyModel, SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)),
-                   this, SLOT(verifyModel(QModelIndex,int,int)));
-        disconnect(m_proxyModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-                   this, SLOT(verifyModel(QModelIndex,int,int)));
-        disconnect(m_proxyModel, SIGNAL(rowsAboutToBeMoved(QModelIndex,int,int,QModelIndex,int)),
-                   this, SLOT(verifyModel(QModelIndex,int,int,QModelIndex,int)));
-        disconnect(m_proxyModel, SIGNAL(rowsMoved(QModelIndex,int,int,QModelIndex,int)),
-                   this, SLOT(verifyModel(QModelIndex,int,int,QModelIndex,int)));
-        disconnect(m_proxyModel, SIGNAL(columnsAboutToBeInserted(QModelIndex,int,int)),
-                   this, SLOT(verifyModel(QModelIndex,int,int)));
-        disconnect(m_proxyModel, SIGNAL(columnsInserted(QModelIndex,int,int)),
-                   this, SLOT(verifyModel(QModelIndex,int,int)));
-        disconnect(m_proxyModel, SIGNAL(columnsAboutToBeRemoved(QModelIndex,int,int)),
-                   this, SLOT(verifyModel(QModelIndex,int,int)));
-        disconnect(m_proxyModel, SIGNAL(columnsRemoved(QModelIndex,int,int)),
-                   this, SLOT(verifyModel(QModelIndex,int,int)));
-        disconnect(m_proxyModel, SIGNAL(columnsAboutToBeMoved(QModelIndex,int,int,QModelIndex,int)),
-                   this, SLOT(verifyModel(QModelIndex,int,int,QModelIndex,int)));
-        disconnect(m_proxyModel, SIGNAL(columnsMoved(QModelIndex,int,int,QModelIndex,int)),
-                   this, SLOT(verifyModel(QModelIndex,int,int,QModelIndex,int)));
-        disconnect(m_proxyModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-                   this, SLOT(verifyModel(QModelIndex,QModelIndex)));
+        disconnect(m_proxyModel, SIGNAL(rowsAboutToBeInserted(QModelIndex, int, int)), this, SLOT(verifyModel(QModelIndex, int, int)));
+        disconnect(m_proxyModel, SIGNAL(rowsInserted(QModelIndex, int, int)), this, SLOT(verifyModel(QModelIndex, int, int)));
+        disconnect(m_proxyModel, SIGNAL(rowsAboutToBeRemoved(QModelIndex, int, int)), this, SLOT(verifyModel(QModelIndex, int, int)));
+        disconnect(m_proxyModel, SIGNAL(rowsRemoved(QModelIndex, int, int)), this, SLOT(verifyModel(QModelIndex, int, int)));
+        disconnect(m_proxyModel,
+                   SIGNAL(rowsAboutToBeMoved(QModelIndex, int, int, QModelIndex, int)),
+                   this,
+                   SLOT(verifyModel(QModelIndex, int, int, QModelIndex, int)));
+        disconnect(m_proxyModel, SIGNAL(rowsMoved(QModelIndex, int, int, QModelIndex, int)), this, SLOT(verifyModel(QModelIndex, int, int, QModelIndex, int)));
+        disconnect(m_proxyModel, SIGNAL(columnsAboutToBeInserted(QModelIndex, int, int)), this, SLOT(verifyModel(QModelIndex, int, int)));
+        disconnect(m_proxyModel, SIGNAL(columnsInserted(QModelIndex, int, int)), this, SLOT(verifyModel(QModelIndex, int, int)));
+        disconnect(m_proxyModel, SIGNAL(columnsAboutToBeRemoved(QModelIndex, int, int)), this, SLOT(verifyModel(QModelIndex, int, int)));
+        disconnect(m_proxyModel, SIGNAL(columnsRemoved(QModelIndex, int, int)), this, SLOT(verifyModel(QModelIndex, int, int)));
+        disconnect(m_proxyModel,
+                   SIGNAL(columnsAboutToBeMoved(QModelIndex, int, int, QModelIndex, int)),
+                   this,
+                   SLOT(verifyModel(QModelIndex, int, int, QModelIndex, int)));
+        disconnect(m_proxyModel,
+                   SIGNAL(columnsMoved(QModelIndex, int, int, QModelIndex, int)),
+                   this,
+                   SLOT(verifyModel(QModelIndex, int, int, QModelIndex, int)));
+        disconnect(m_proxyModel, SIGNAL(dataChanged(QModelIndex, QModelIndex)), this, SLOT(verifyModel(QModelIndex, QModelIndex)));
     }
 
     m_proxyModel = proxyModel;
@@ -470,52 +466,29 @@ void ProxyModelTest::connectProxy(QAbstractProxyModel *proxyModel)
     QVERIFY(m_modelSpy->isEmpty());
     testMappings();
 
-    connect(m_proxyModel, SIGNAL(rowsAboutToBeInserted(QModelIndex,int,int)),
-            SLOT(testMappings()));
-    connect(m_proxyModel, SIGNAL(rowsInserted(QModelIndex,int,int)),
-            SLOT(testMappings()));
-    connect(m_proxyModel, SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)),
-            SLOT(testMappings()));
-    connect(m_proxyModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-            SLOT(testMappings()));
-    connect(m_proxyModel, SIGNAL(layoutAboutToBeChanged()),
-            SLOT(testMappings()));
-    connect(m_proxyModel, SIGNAL(layoutChanged()),
-            SLOT(testMappings()));
-    connect(m_proxyModel, SIGNAL(rowsAboutToBeMoved(QModelIndex,int,int,QModelIndex,int)),
-            SLOT(testMappings()));
-    connect(m_proxyModel, SIGNAL(rowsMoved(QModelIndex,int,int,QModelIndex,int)),
-            SLOT(testMappings()));
-    connect(m_proxyModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-            SLOT(testMappings()));
-
-    connect(m_proxyModel, SIGNAL(rowsAboutToBeInserted(QModelIndex,int,int)),
-            SLOT(verifyModel(QModelIndex,int,int)));
-    connect(m_proxyModel, SIGNAL(rowsInserted(QModelIndex,int,int)),
-            SLOT(verifyModel(QModelIndex,int,int)));
-    connect(m_proxyModel, SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)),
-            SLOT(verifyModel(QModelIndex,int,int)));
-    connect(m_proxyModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-            SLOT(verifyModel(QModelIndex,int,int)));
-    connect(m_proxyModel, SIGNAL(rowsAboutToBeMoved(QModelIndex,int,int,QModelIndex,int)),
-            SLOT(verifyModel(QModelIndex,int,int,QModelIndex,int)));
-    connect(m_proxyModel, SIGNAL(rowsMoved(QModelIndex,int,int,QModelIndex,int)),
-            SLOT(verifyModel(QModelIndex,int,int,QModelIndex,int)));
-    connect(m_proxyModel, SIGNAL(columnsAboutToBeInserted(QModelIndex,int,int)),
-            SLOT(verifyModel(QModelIndex,int,int)));
-    connect(m_proxyModel, SIGNAL(columnsInserted(QModelIndex,int,int)),
-            SLOT(verifyModel(QModelIndex,int,int)));
-    connect(m_proxyModel, SIGNAL(columnsAboutToBeRemoved(QModelIndex,int,int)),
-            SLOT(verifyModel(QModelIndex,int,int)));
-    connect(m_proxyModel, SIGNAL(columnsRemoved(QModelIndex,int,int)),
-            SLOT(verifyModel(QModelIndex,int,int)));
-    connect(m_proxyModel, SIGNAL(columnsAboutToBeMoved(QModelIndex,int,int,QModelIndex,int)),
-            SLOT(verifyModel(QModelIndex,int,int,QModelIndex,int)));
-    connect(m_proxyModel, SIGNAL(columnsMoved(QModelIndex,int,int,QModelIndex,int)),
-            SLOT(verifyModel(QModelIndex,int,int,QModelIndex,int)));
-    connect(m_proxyModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-            SLOT(verifyModel(QModelIndex,QModelIndex)));
+    connect(m_proxyModel, SIGNAL(rowsAboutToBeInserted(QModelIndex, int, int)), SLOT(testMappings()));
+    connect(m_proxyModel, SIGNAL(rowsInserted(QModelIndex, int, int)), SLOT(testMappings()));
+    connect(m_proxyModel, SIGNAL(rowsAboutToBeRemoved(QModelIndex, int, int)), SLOT(testMappings()));
+    connect(m_proxyModel, SIGNAL(rowsRemoved(QModelIndex, int, int)), SLOT(testMappings()));
+    connect(m_proxyModel, SIGNAL(layoutAboutToBeChanged()), SLOT(testMappings()));
+    connect(m_proxyModel, SIGNAL(layoutChanged()), SLOT(testMappings()));
+    connect(m_proxyModel, SIGNAL(rowsAboutToBeMoved(QModelIndex, int, int, QModelIndex, int)), SLOT(testMappings()));
+    connect(m_proxyModel, SIGNAL(rowsMoved(QModelIndex, int, int, QModelIndex, int)), SLOT(testMappings()));
+    connect(m_proxyModel, SIGNAL(dataChanged(QModelIndex, QModelIndex)), SLOT(testMappings()));
 
+    connect(m_proxyModel, SIGNAL(rowsAboutToBeInserted(QModelIndex, int, int)), SLOT(verifyModel(QModelIndex, int, int)));
+    connect(m_proxyModel, SIGNAL(rowsInserted(QModelIndex, int, int)), SLOT(verifyModel(QModelIndex, int, int)));
+    connect(m_proxyModel, SIGNAL(rowsAboutToBeRemoved(QModelIndex, int, int)), SLOT(verifyModel(QModelIndex, int, int)));
+    connect(m_proxyModel, SIGNAL(rowsRemoved(QModelIndex, int, int)), SLOT(verifyModel(QModelIndex, int, int)));
+    connect(m_proxyModel, SIGNAL(rowsAboutToBeMoved(QModelIndex, int, int, QModelIndex, int)), SLOT(verifyModel(QModelIndex, int, int, QModelIndex, int)));
+    connect(m_proxyModel, SIGNAL(rowsMoved(QModelIndex, int, int, QModelIndex, int)), SLOT(verifyModel(QModelIndex, int, int, QModelIndex, int)));
+    connect(m_proxyModel, SIGNAL(columnsAboutToBeInserted(QModelIndex, int, int)), SLOT(verifyModel(QModelIndex, int, int)));
+    connect(m_proxyModel, SIGNAL(columnsInserted(QModelIndex, int, int)), SLOT(verifyModel(QModelIndex, int, int)));
+    connect(m_proxyModel, SIGNAL(columnsAboutToBeRemoved(QModelIndex, int, int)), SLOT(verifyModel(QModelIndex, int, int)));
+    connect(m_proxyModel, SIGNAL(columnsRemoved(QModelIndex, int, int)), SLOT(verifyModel(QModelIndex, int, int)));
+    connect(m_proxyModel, SIGNAL(columnsAboutToBeMoved(QModelIndex, int, int, QModelIndex, int)), SLOT(verifyModel(QModelIndex, int, int, QModelIndex, int)));
+    connect(m_proxyModel, SIGNAL(columnsMoved(QModelIndex, int, int, QModelIndex, int)), SLOT(verifyModel(QModelIndex, int, int, QModelIndex, int)));
+    connect(m_proxyModel, SIGNAL(dataChanged(QModelIndex, QModelIndex)), SLOT(verifyModel(QModelIndex, QModelIndex)));
 }
 
 void ProxyModelTest::doTest()
@@ -532,8 +505,7 @@ void ProxyModelTest::doTest()
         return;
     }
 
-    if ((signalList.size() == 1 && signalList.first().size() == 1)
-            && signalList.first().first().toString() == QLatin1String("skip")) {
+    if ((signalList.size() == 1 && signalList.first().size() == 1) && signalList.first().first().toString() == QLatin1String("skip")) {
         return;
     }
 
@@ -635,9 +607,8 @@ void ProxyModelTest::connectTestSignals(QObject *receiver)
     }
     for (int methodIndex = 0; methodIndex < metaObject()->methodCount(); ++methodIndex) {
         QMetaMethod mm = metaObject()->method(methodIndex);
-        if (mm.methodType() == QMetaMethod::Signal
-                && QString(mm.methodSignature()).startsWith(QLatin1String("test"))
-                && QString(mm.methodSignature()).endsWith(QLatin1String("Data()"))) {
+        if (mm.methodType() == QMetaMethod::Signal && QString(mm.methodSignature()).startsWith(QLatin1String("test"))
+            && QString(mm.methodSignature()).endsWith(QLatin1String("Data()"))) {
             int slotIndex = receiver->metaObject()->indexOfSlot(mm.methodSignature());
             Q_ASSERT(slotIndex >= 0);
             metaObject()->connect(this, methodIndex, receiver, slotIndex);
@@ -652,9 +623,8 @@ void ProxyModelTest::disconnectTestSignals(QObject *receiver)
     }
     for (int methodIndex = 0; methodIndex < metaObject()->methodCount(); ++methodIndex) {
         QMetaMethod mm = metaObject()->method(methodIndex);
-        if (mm.methodType() == QMetaMethod::Signal
-                && QString(mm.methodSignature()).startsWith(QLatin1String("test"))
-                && QString(mm.methodSignature()).endsWith(QLatin1String("Data()"))) {
+        if (mm.methodType() == QMetaMethod::Signal && QString(mm.methodSignature()).startsWith(QLatin1String("test"))
+            && QString(mm.methodSignature()).endsWith(QLatin1String("Data()"))) {
             int slotIndex = receiver->metaObject()->indexOfSlot(mm.methodSignature());
             Q_ASSERT(slotIndex >= 0);
             metaObject()->disconnect(this, methodIndex, receiver, slotIndex);
@@ -694,7 +664,9 @@ uint qHash(const QVariant &var)
     case QVariant::DateTime:
     case QVariant::Url:
     case QVariant::Locale:
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     case QVariant::RegExp:
+#endif
         return qHash(var.toString());
     case QVariant::Map:
     case QVariant::List:
