@@ -755,12 +755,12 @@ void KSelectionProxyModelPrivate::sourceLayoutAboutToBeChanged()
     QItemSelection selection;
     for (const QModelIndex &rootIndex : std::as_const(m_rootIndexList)) {
         // This will be optimized later.
-        Q_EMIT q->rootIndexAboutToBeRemoved(rootIndex, {});
+        Q_EMIT q->rootIndexAboutToBeRemoved(rootIndex, KSelectionProxyModel::QPrivateSignal());
         selection.append(QItemSelectionRange(rootIndex, rootIndex));
     }
 
     selection = kNormalizeSelection(selection);
-    Q_EMIT q->rootSelectionAboutToBeRemoved(selection, {});
+    Q_EMIT q->rootSelectionAboutToBeRemoved(selection, KSelectionProxyModel::QPrivateSignal());
 
     QPersistentModelIndex srcPersistentIndex;
     const auto lst = q->persistentIndexList();
@@ -1723,7 +1723,7 @@ void KSelectionProxyModelPrivate::removeSelectionFromProxy(const QItemSelection
         if (!rootWillBeRemoved(selection, *rootIt)) {
             break;
         }
-        q->rootIndexAboutToBeRemoved(*rootIt, {});
+        q->rootIndexAboutToBeRemoved(*rootIt, KSelectionProxyModel::QPrivateSignal());
         if (m_startWithChildTrees) {
             auto rc = q->sourceModel()->rowCount(*rootIt);
             proxyEndRemove += rc;
@@ -1940,7 +1940,7 @@ void KSelectionProxyModelPrivate::insertSelectionIntoProxy(const QItemSelection
                 // We still need to make sure its future children are inserted into the model.
                 m_rootIndexList.insert(rootListRow, newIndex);
                 if (!m_resetting || m_layoutChanging) {
-                    Q_EMIT q->rootIndexAdded(newIndex, {});
+                    Q_EMIT q->rootIndexAdded(newIndex, KSelectionProxyModel::QPrivateSignal());
                 }
                 continue;
             }
@@ -1950,7 +1950,7 @@ void KSelectionProxyModelPrivate::insertSelectionIntoProxy(const QItemSelection
             Q_ASSERT(newIndex.isValid());
             m_rootIndexList.insert(rootListRow, newIndex);
             if (!m_resetting || m_layoutChanging) {
-                Q_EMIT q->rootIndexAdded(newIndex, {});
+                Q_EMIT q->rootIndexAdded(newIndex, KSelectionProxyModel::QPrivateSignal());
             }
 
             int _start = 0;
@@ -1976,7 +1976,7 @@ void KSelectionProxyModelPrivate::insertSelectionIntoProxy(const QItemSelection
             m_rootIndexList.insert(row, newIndex);
 
             if (!m_resetting || m_layoutChanging) {
-                Q_EMIT q->rootIndexAdded(newIndex, {});
+                Q_EMIT q->rootIndexAdded(newIndex, KSelectionProxyModel::QPrivateSignal());
             }
             Q_ASSERT(m_rootIndexList.size() > row);
             updateInternalIndexes(QModelIndex(), row, 1);
@@ -1987,7 +1987,7 @@ void KSelectionProxyModelPrivate::insertSelectionIntoProxy(const QItemSelection
             }
         }
     }
-    q->rootSelectionAdded(selection, {});
+    Q_EMIT q->rootSelectionAdded(selection, KSelectionProxyModel::QPrivateSignal());
 }
 
 KSelectionProxyModel::KSelectionProxyModel(QItemSelectionModel *selectionModel, QObject *parent)
@@ -2057,7 +2057,7 @@ void KSelectionProxyModel::setFilterBehavior(FilterBehavior behavior)
             break;
         }
         }
-        Q_EMIT filterBehaviorChanged({});
+        Q_EMIT filterBehaviorChanged(QPrivateSignal());
         d->resetInternalData();
         if (d->m_selectionModel) {
             d->selectionChanged(d->m_selectionModel->selection(), QItemSelection());
@@ -2381,7 +2381,7 @@ void KSelectionProxyModel::setSelectionModel(QItemSelectionModel *itemSelectionM
         }
 
         d->m_selectionModel = itemSelectionModel;
-        Q_EMIT selectionModelChanged({});
+        Q_EMIT selectionModelChanged(QPrivateSignal());
 
         if (d->m_selectionModel) {
             connect(d->m_selectionModel, SIGNAL(selectionChanged(QItemSelection, QItemSelection)), SLOT(selectionChanged(QItemSelection, QItemSelection)));
